// 
//  array.c
//  mivip
//  
//  A simple multi-dimensial array data-structure.
//
//  Created by Maarten Everts.
// 
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>
#include "sarray.h"

SArray *SArray_new(void)
{
    return (SArray*)malloc(sizeof(SArray));
}


int SArray_getNumItems(SArray *self)
{
    int numItems = 1;
    for(size_t i = 0; i < self->nd; ++i) {
        numItems *= self->shape[i];
    }
    return numItems;    
}
void SArray_allocateMem(SArray *self)
{
    int numItems = SArray_getNumItems(self);
    self->data = malloc(self->itemSize * numItems);
}

void SArray_init(SArray *self, int nd, int *shape, int itemSize)
{
    self->nd = nd;
    memcpy(self->shape,shape,nd*sizeof(int));
    self->itemSize = itemSize;
    self->strides[self->nd - 1] = 1;
    for(size_t i = 1; i < self->nd; ++i) {
        self->strides[self->nd - 1 - i] = self->strides[self->nd - i] * self->shape[self->nd - i];
    }
    self->ownData = 1;

    SArray_allocateMem(self);
}

void SArray_initAsView(SArray *self, int nd, int *shape, int itemSize, void *data)
{
    self->nd = nd;
    memcpy(self->shape,shape,nd*sizeof(int));
    self->itemSize = itemSize;
    self->strides[self->nd - 1] = 1;
    for(size_t i = 1; i < self->nd; ++i) {
        self->strides[self->nd - 1 - i] = self->strides[self->nd - i] * self->shape[self->nd - i];
    }
    self->ownData = 0;
    self->data = data;
}

void SArray_initCopy(SArray *self, SArray *other)
{
    SArray_init(self,other->nd,other->shape,other->itemSize);
    int numItems = SArray_getNumItems(self);
    if (numItems > 0) {
        memcpy(self->data,other->data,self->itemSize*numItems);
    }
}

void SArray_freeData(SArray *self)
{
    if (self->data) {
        if (self->ownData) {
            free(self->data);
        }
        self->nd = 0;
        self->data = NULL;
    }
}

void SArray_clear(SArray *self)
{
    bzero(self->data,self->itemSize*SArray_getNumItems(self));
}

void SArray_free(SArray *self)
{
    if (self) {
        SArray_freeData(self);
        free(self);
    }
}

void printIntArray(int numItems, int *intArray)
{
    printf("[");
    if (numItems > 0) {
        printf("%d",intArray[0]);
        for(size_t i = 1; i < numItems; ++i) {
            printf(", %d",intArray[i]);
        }
    }
    printf("]");
}

void SArray_printInfo(SArray *self)
{
    printf("nd: %d\n",self->nd);
    printf("shape: ");
    printIntArray(self->nd,self->shape);
    printf("\nstrides: ");
    printIntArray(self->nd,self->strides);
    printf("\nitemSize: %d\n",self->itemSize);
}
