#!/usr/bin/env python
# encoding: utf-8
"""
sldfile.py

A simple module for reading and writing .sld files. A .sld file is
a simple text file to store lines in. Note there is very little
input validation, so be careful.

Created by Maarten Everts on 2009-05-15.
Copyright (c) 2009 University of Groningen. All rights reserved.
"""

import numpy

def read(filename):
    """Read a .sld file to a list of numpy arrays."""
    infile = open(filename,'r')
    textdata = infile.readlines()
    # The first line is the 'header'. It has the following form:
    # <num_lines> <dim> <num_attr>
    # The last two are optional and if absent default to 3 and 0 respectively. 
    header = [int(x) for x in textdata[0].strip().split(" ")]
    num_lines = header[0]
    if len(header)==3:
        dim = header[1]
        num_attr = header[2]
    else:
        dim = 3
        num_attr = 0
    # A bit of input validation
    assert type(dim) is int, "Expected integer for dim."
    assert type(num_attr) is int, "Expected integer for num_attr."
    linedata = []
    current_pos = 1
    for i in xrange(num_lines):
        num_points = int(textdata[current_pos].strip())
        linedata.append(numpy.array(
                        [[float(c) for c in p.strip().split(' ')] for p in 
                        textdata[current_pos+1:current_pos+1+num_points]],numpy.float32))
        current_pos += num_points + 1
    return linedata

def write(linedata, filename):
    """Write linedata (list of numpy arrays) to a .sld file."""
    outfile = open(filename, 'w')
    num_lines = len(linedata)
    dim = linedata[0].shape[1] # take the first line as indicative
    num_attr = linedata[0].shape[1] - dim
    outfile.write(str(num_lines) + " " + str(dim) + " " + str(num_attr) + "\n")
    for line in linedata:
        assert line.shape[1] == dim, "Numpy array has unexpected shape."
        outfile.write(str(line.shape[0])+"\n")
        for point in line:
            outfile.write(' '.join([str(c) for c in point])+"\n")
